@@ -34,11 +34,14 @@
 **  RCS:
 **
 **      $Source: /afs/pitt.edu/usr12/dgm/work/IMAP_Proxy/src/RCS/pimpstat.c,v $
-**      $Id: pimpstat.c,v 1.7 2004/02/24 14:56:10 dgm Exp $
+**      $Id: pimpstat.c,v 1.8 2005/06/15 12:11:36 dgm Exp $
 **      
 **  Modification History:
 **
 **      $Log: pimpstat.c,v $
+**      Revision 1.8  2005/06/15 12:11:36  dgm
+**      Patch by Mathew Anderson to add -c flag and behavior.
+**
 **      Revision 1.7  2004/02/24 14:56:10  dgm
 **      Added SELECT cache stuff.
 **
@@ -83,6 +86,10 @@
 #include <sys/param.h>
 #endif
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #define DIGITS 11
 
 extern WINDOW *stdscr;
@@ -144,7 +151,7 @@ int main( int argc, char *argv[] )
     IMAPCounter_Struct *IMAPCount;
     int fd;
     char *fn = "pimpstat";
-    int i, rc;
+    int i, command;
     char ccc[DIGITS+1];  /* current client conns */
     char pcc[DIGITS+1];  /* peak client conns */
     char asc[DIGITS+1];  /* active server conns */
@@ -169,8 +176,10 @@ int main( int argc, char *argv[] )
     ConfigFile[0] = '\0';
     
     signal( SIGINT, (void (*)()) Handler );
-
-    while (( i = getopt( argc, argv, "f:h" ) ) != EOF )
+    
+    command = 0;
+    
+    while (( i = getopt( argc, argv, "f:ch" ) ) != EOF )
     {
 	
         switch( i )
@@ -180,7 +189,12 @@ int main( int argc, char *argv[] )
             /* user specified a config filename */
             strncpy( ConfigFile, optarg, sizeof ConfigFile -1 );
             break;
-            
+    
+	case 'c':
+	    /* user wants output via command line */
+	    command=1;
+	    break;
+	    
         case 'h':
             Usage();
             exit( 0 );
@@ -221,124 +235,149 @@ int main( int argc, char *argv[] )
     }
     
 
-    stdscr = initscr();
+    if ( command == 0 )
+    {
+	stdscr = initscr();
 
-    if ( !stdscr )
-    {
-	printf("%s: failed to initialize screen -- exiting.\n", fn );
-	exit( 1 );
-    }
-    
-    border( 0, 0, 0, 0, 0, 0, 0, 0 );
-    mvaddstr( 2, 8, "Server Start Time:" );
-    mvaddstr( 3, 2, "Last Counter Reset Time:" );
-    mvaddstr( 5, 2, "CLIENT CONNECTIONS" );
-    mvaddstr( 7, 5, "current:" );
-    mvaddstr( 7, 40, "peak:" );
-    mvaddstr( 9, 2, "ACTIVE SERVER CONNECTIONS" );
-    mvaddstr( 11, 5, "current:" );
-    mvaddstr( 11, 40, "peak:" );
-    mvaddstr( 13, 2, "CACHED SERVER CONNECTIONS" );
-    mvaddstr( 15, 5, "current:");
-    mvaddstr( 15, 40, "peak:" );
-    mvaddstr( 17, 2, "CONNECTION TOTALS" );
-    mvaddstr( 19, 5, "client connections accepted:" );
-    mvaddstr( 20, 5, "client logins:" );
-    mvaddstr( 21, 5, "server connections created:" );
-    mvaddstr( 22, 5, "server connection reuses:" );
-    mvaddstr( 23, 5, "client login to server login ratio:" );
-    if ( PC_Struct.enable_select_cache )
-    {
-	mvaddstr( 25, 2, "SELECT CACHE TOTALS" );
-	mvaddstr( 27, 5, "hit:" );
-	mvaddstr( 27, 40, "miss:" );
-    }
-    else
-    {
-	mvaddstr( 25, 2, "SELECT CACHE NOT ENABLED" );
-    }
-    
-    mvaddstr( 29, 2, "CTRL-C to quit." );
-
-    for ( ; ; )
-    {
-	/*
-	 * I don't know crap about curses.  There's prolly an easy way to 
-	 * accomplish this, but I don't know how.  Basically we have to
-	 * turn all of our numbers into strings so curses can display them.
-	 * I'd guess there's a printf equivalent in curses, but I dunno.
-	 */
-
-	if ( IMAPCount->TotalServerConnectionsCreated == 0 )
+	if ( !stdscr )
 	{
-	    snprintf( ssrr, DIGITS + 3, "          N/A" );
+	    printf("%s: failed to initialize screen -- exiting.\n", fn );
+	    exit( 1 );
+	}
+	
+	border( 0, 0, 0, 0, 0, 0, 0, 0 );
+	mvaddstr( 2, 8, "Server Start Time:" );
+	mvaddstr( 3, 2, "Last Counter Reset Time:" );
+	mvaddstr( 5, 2, "CLIENT CONNECTIONS" );
+	mvaddstr( 7, 5, "current:" );
+	mvaddstr( 7, 40, "peak:" );
+	mvaddstr( 9, 2, "ACTIVE SERVER CONNECTIONS" );
+	mvaddstr( 11, 5, "current:" );
+	mvaddstr( 11, 40, "peak:" );
+	mvaddstr( 13, 2, "CACHED SERVER CONNECTIONS" );
+	mvaddstr( 15, 5, "current:");
+	mvaddstr( 15, 40, "peak:" );
+	mvaddstr( 17, 2, "CONNECTION TOTALS" );
+	mvaddstr( 19, 5, "client connections accepted:" );
+	mvaddstr( 20, 5, "client logins:" );
+	mvaddstr( 21, 5, "server connections created:" );
+	mvaddstr( 22, 5, "server connection reuses:" );
+	mvaddstr( 23, 5, "client login to server login ratio:" );
+	if ( PC_Struct.enable_select_cache )
+	{
+	    mvaddstr( 25, 2, "SELECT CACHE TOTALS" );
+	    mvaddstr( 27, 5, "hit:" );
+	    mvaddstr( 27, 40, "miss:" );
 	}
 	else
 	{
-	    Ratio = (float)IMAPCount->TotalClientLogins /
-		(float)IMAPCount->TotalServerConnectionsCreated;
-	    snprintf( ssrr, DIGITS + 3, "%9.2f : 1", Ratio );
+	    mvaddstr( 25, 2, "SELECT CACHE NOT ENABLED" );
 	}
 	
-	/*
-	 * ctime is putting a \n at the end of the string and that's
-	 * making curses do goofy stuff that I don't understand.  Rather
-	 * than figure out why it's breaking curses, I'm just going to
-	 * copy ctime's strings into my own buffers and get rid of the
-	 * \n.
-	 */
-	strncpy( stimebuf, ctime( &IMAPCount->StartTime ),
-		 sizeof stimebuf - 1 );
-	strncpy( ctimebuf, ctime( &IMAPCount->CountTime ),
-		 sizeof ctimebuf - 1 );
-
-	CP = strrchr( stimebuf, '\n' );
-	if (CP)
-	    *CP = '\0';
-
-	CP = strrchr( ctimebuf, '\n' );
-	if (CP)
-	    *CP ='\0';
+	mvaddstr( 29, 2, "CTRL-C to quit." );
 	
-	snprintf( ccc, DIGITS, "%9d", IMAPCount->CurrentClientConnections );
-	snprintf( pcc, DIGITS, "%9d", IMAPCount->PeakClientConnections );
-	snprintf( asc, DIGITS, "%9d", IMAPCount->InUseServerConnections );
-	snprintf( psc, DIGITS, "%9d", IMAPCount->PeakInUseServerConnections );
-	snprintf( rsc, DIGITS, "%9d", IMAPCount->RetainedServerConnections );
-	snprintf( prsc, DIGITS, "%9d", IMAPCount->PeakRetainedServerConnections );
-	snprintf( tcca, DIGITS, "%9d", IMAPCount->TotalClientConnectionsAccepted );
-	snprintf( tcl, DIGITS, "%9d", IMAPCount->TotalClientLogins );
-	snprintf( tscr, DIGITS, "%9d", IMAPCount->TotalServerConnectionsReused );
-	snprintf( tscc, DIGITS, "%9d", IMAPCount->TotalServerConnectionsCreated );
-	snprintf( tsch, DIGITS, "%9d", IMAPCount->SelectCacheHits );
-	snprintf( tscm, DIGITS, "%9d", IMAPCount->SelectCacheMisses );
-	
-	mvaddstr( 2, 31, stimebuf );
-	mvaddstr( 3, 31, ctimebuf );
-	mvaddstr( 7, 14, ccc );
-	mvaddstr( 7, 46, pcc );
-	mvaddstr( 11, 14, asc );
-	mvaddstr( 11, 46, psc );
-	mvaddstr( 15, 14, rsc );
-	mvaddstr( 15, 46, prsc );
-	mvaddstr( 19, 46, tcca );
-	mvaddstr( 20, 46, tcl );
-	mvaddstr( 21, 46, tscc );
-	mvaddstr( 22, 46, tscr );
-	mvaddstr( 23, 42, ssrr );
-	if ( PC_Struct.enable_select_cache )
+	for ( ; ; )
 	{
-	    mvaddstr( 27, 14, tsch );
-	    mvaddstr( 27, 46, tscm );
+	    /*
+	     * I don't know crap about curses.  There's prolly an easy way to 
+	     * accomplish this, but I don't know how.  Basically we have to
+	     * turn all of our numbers into strings so curses can display them.
+	     * I'd guess there's a printf equivalent in curses, but I dunno.
+	     */
+	    
+	    if ( IMAPCount->TotalServerConnectionsCreated == 0 )
+	    {
+		snprintf( ssrr, DIGITS + 3, "          N/A" );
+	    }
+	    else
+	    {
+		Ratio = (float)IMAPCount->TotalClientLogins /
+		    (float)IMAPCount->TotalServerConnectionsCreated;
+		snprintf( ssrr, DIGITS + 3, "%9.2f : 1", Ratio );
+	    }
+	    
+	    /*
+	     * ctime is putting a \n at the end of the string and that's
+	     * making curses do goofy stuff that I don't understand.  Rather
+	     * than figure out why it's breaking curses, I'm just going to
+	     * copy ctime's strings into my own buffers and get rid of the
+	     * \n.
+	     */
+	    strncpy( stimebuf, ctime( &IMAPCount->StartTime ),
+		     sizeof stimebuf - 1 );
+	    strncpy( ctimebuf, ctime( &IMAPCount->CountTime ),
+		     sizeof ctimebuf - 1 );
+	    
+	    CP = strrchr( stimebuf, '\n' );
+	    if (CP)
+		*CP = '\0';
+	    
+	    CP = strrchr( ctimebuf, '\n' );
+	    if (CP)
+		*CP ='\0';
+	    
+	    snprintf( ccc, DIGITS, "%9d", IMAPCount->CurrentClientConnections );
+	    snprintf( pcc, DIGITS, "%9d", IMAPCount->PeakClientConnections );
+	    snprintf( asc, DIGITS, "%9d", IMAPCount->InUseServerConnections );
+	    snprintf( psc, DIGITS, "%9d", IMAPCount->PeakInUseServerConnections );
+	    snprintf( rsc, DIGITS, "%9d", IMAPCount->RetainedServerConnections );
+	    snprintf( prsc, DIGITS, "%9d", IMAPCount->PeakRetainedServerConnections );
+	    snprintf( tcca, DIGITS, "%9d", IMAPCount->TotalClientConnectionsAccepted );
+	    snprintf( tcl, DIGITS, "%9d", IMAPCount->TotalClientLogins );
+	    snprintf( tscr, DIGITS, "%9d", IMAPCount->TotalServerConnectionsReused );
+	    snprintf( tscc, DIGITS, "%9d", IMAPCount->TotalServerConnectionsCreated );
+	    snprintf( tsch, DIGITS, "%9d", IMAPCount->SelectCacheHits );
+	    snprintf( tscm, DIGITS, "%9d", IMAPCount->SelectCacheMisses );
+	    
+	    mvaddstr( 2, 31, stimebuf );
+	    mvaddstr( 3, 31, ctimebuf );
+	    mvaddstr( 7, 14, ccc );
+	    mvaddstr( 7, 46, pcc );
+	    mvaddstr( 11, 14, asc );
+	    mvaddstr( 11, 46, psc );
+	    mvaddstr( 15, 14, rsc );
+	    mvaddstr( 15, 46, prsc );
+	    mvaddstr( 19, 46, tcca );
+	    mvaddstr( 20, 46, tcl );
+	    mvaddstr( 21, 46, tscc );
+	    mvaddstr( 22, 46, tscr );
+	    mvaddstr( 23, 42, ssrr );
+	    if ( PC_Struct.enable_select_cache )
+	    {
+		mvaddstr( 27, 14, tsch );
+		mvaddstr( 27, 46, tscm );
+	    }
+	    
+	    refresh();
+	    
+	    sleep( 1 );
 	}
-	   
-	refresh();
 	
-        sleep( 1 );
+    }
+    else
+    {
+	/*
+	 * We only get here if command is non-zero.
+	 */
+	printf( " %d Current Client Connections\n %d Peak Client Connections\n %d In Use Connections\n %d Peak In Use Connections\n %d Retained Server Connections\n %d Peak Retained Server Connections\n %d Total Client Connectsions\n %d Total Client Logins\n %d Total Reused Connections\n %d Total Created Connections\n %d Cache Hits\n %d Cache Misses\n", IMAPCount->CurrentClientConnections,
+		IMAPCount->PeakClientConnections, 
+		IMAPCount->InUseServerConnections,
+		IMAPCount->PeakInUseServerConnections,
+		IMAPCount->RetainedServerConnections,
+		IMAPCount->PeakRetainedServerConnections,
+		IMAPCount->TotalClientConnectionsAccepted,
+		IMAPCount->TotalClientLogins,
+		IMAPCount->TotalServerConnectionsReused,
+		IMAPCount->TotalServerConnectionsCreated, 
+		IMAPCount->SelectCacheHits,
+		IMAPCount->SelectCacheMisses );
+
+	exit( 0 );
     }
 }
 
 
+
 /*++
  * Function:    Usage
  *
@@ -355,7 +394,9 @@ int main( int argc, char *argv[] )
  */
 void Usage( void )
 {
-    printf("Usage: pimpstat [-f config filename] [-h]\n");
+    printf( "Usage: pimpstat [-f config filename] [-h] [-c]\n" );
+    printf( " -c is for simple command line output format instead of curses.\n" );
+    
     return;
 }
 
