@@ -39,11 +39,18 @@
 **  RCS:
 **
 **	$Source: /afs/pitt.edu/usr12/dgm/work/IMAP_Proxy/src/RCS/request.c,v $
-**	$Id: request.c,v 1.19 2004/11/10 15:33:04 dgm Exp $
+**	$Id: request.c,v 1.21 2005/07/06 11:53:19 dgm Exp $
 **      
 **  Modification History:
 **
 **	$Log: request.c,v $
+**	Revision 1.21  2005/07/06 11:53:19  dgm
+**	Added support for enable_admin_commands config option.
+**
+**	Revision 1.20  2005/06/15 12:07:12  dgm
+**	Remove unused variables.  Fixed snprintf argument lists.
+**	Include missing config.h directive.
+**
 **	Revision 1.19  2004/11/10 15:33:04  dgm
 **	Explictly NULL terminate all strings that are the result
 **	of strncpy.  Also enforce checking of LiteralBytesRemaining
@@ -123,10 +130,10 @@
 
 #define _REENTRANT
 
+#include <config.h>
+
 #include <errno.h>
 #include <string.h>
-#include "common.h"
-#include "imapproxy.h"
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/uio.h>
@@ -145,6 +152,9 @@
 
 #include <openssl/evp.h>
 
+#include "common.h"
+#include "imapproxy.h"
+
 /*
  * There are a few global variables that we care about.  Make sure we know
  * about them here.
@@ -204,6 +214,18 @@ static int cmd_newlog( ITD_Struct *itd, char *Tag )
     int rc;
     
     SendBuf[BUFSIZE - 1] = '\0';
+    
+    if ( ! PC_Struct.enable_admin_commands )
+    {
+	snprintf( SendBuf, BufLen, "%s BAD Unrecognized command\r\n", Tag );
+	if ( IMAP_Write( itd->conn, SendBuf, strlen( SendBuf ) ) == -1 )
+	{
+	    syslog( LOG_WARNING, "%s: IMAP_Write() failed: %s", fn, strerror( errno ) );
+	    return( -1 );
+	}
+	return( 0 );
+    }
+    
 
     rc = ftruncate( Tracefd, 0 );
     
@@ -280,6 +302,17 @@ static int cmd_resetcounters( ITD_Struct *itd, char *Tag )
     
     SendBuf[BufLen] = '\0';
 
+    if ( ! PC_Struct.enable_admin_commands )
+    {
+	snprintf( SendBuf, BufLen, "%s BAD Unrecognized command\r\n", Tag );
+	if ( IMAP_Write( itd->conn, SendBuf, strlen( SendBuf ) ) == -1 )
+	{
+	    syslog( LOG_WARNING, "%s: IMAP_Write() failed: %s", fn, strerror( errno ) );
+	    return( -1 );
+	}
+	return( 0 );
+    }
+    
     /*
      * Bugfix by Geoffrey Hort <g.hort@unsw.edu.au> -- I forgot to zero
      * out TotalClientLogins...
@@ -329,6 +362,17 @@ static int cmd_dumpicc( ITD_Struct *itd, char *Tag )
     unsigned int BufLen = BUFSIZE - 1;
     
     SendBuf[BUFSIZE - 1] = '\0';
+
+    if ( ! PC_Struct.enable_admin_commands )
+    {
+	snprintf( SendBuf, BufLen, "%s BAD Unrecognized command\r\n", Tag );
+	if ( IMAP_Write( itd->conn, SendBuf, strlen( SendBuf ) ) == -1 )
+	{
+	    syslog( LOG_WARNING, "%s: IMAP_Write() failed: %s", fn, strerror( errno ) );
+	    return( -1 );
+	}
+	return( 0 );
+    }
     
     LockMutex( &mp );
     
@@ -388,6 +432,17 @@ static int cmd_trace( ITD_Struct *itd, char *Tag, char *Username )
     unsigned int BufLen = BUFSIZE - 1;
     
     SendBuf[BUFSIZE - 1] = '\0';
+
+    if ( ! PC_Struct.enable_admin_commands )
+    {
+	snprintf( SendBuf, BufLen, "%s BAD Unrecognized command\r\n", Tag );
+	if ( IMAP_Write( itd->conn, SendBuf, strlen( SendBuf ) ) == -1 )
+	{
+	    syslog( LOG_WARNING, "%s: IMAP_Write() failed: %s", fn, strerror( errno ) );
+	    return( -1 );
+	}
+	return( 0 );
+    }
     
     /*
      * Here are the tracing semantics:
@@ -819,7 +874,6 @@ static int cmd_login( ITD_Struct *Client,
     ITD_Struct Server;
     int rc;
     ICD_Struct *conn;
-    char TraceFileName[ MAXPATHLEN ];
     struct sockaddr_in cli_addr;
     int addrlen;
     char *hostaddr;
@@ -1393,7 +1447,6 @@ extern void HandleRequest( int clientsd )
     char *Tag;
     char *Command;
     char *Username;
-    char *Password;
     char *AuthMech;
     char *Lasts;
     char *EndOfLine;
@@ -1628,7 +1681,7 @@ extern void HandleRequest( int clientsd )
 		/*
 		 * an auth mechanism we can't handle.
 		 */
-		snprintf( SendBuf, BufLen, "%s NO no mechanism available\r\n", Tag, Command );
+		snprintf( SendBuf, BufLen, "%s NO no mechanism available\r\n", Tag );
 		if ( IMAP_Write( Client.conn, SendBuf, strlen(SendBuf) ) == -1 )
 		{
 		    IMAPCount->CurrentClientConnections--;
@@ -1890,7 +1943,7 @@ extern void HandleRequest( int clientsd )
 		return;
 	    }
 	    
-	    snprintf( SendBuf, BufLen, "%s BAD Please login first\r\n", Tag, Command );
+	    snprintf( SendBuf, BufLen, "%s BAD Please login first\r\n", Tag );
 	    if ( IMAP_Write( Client.conn, SendBuf, strlen(SendBuf) ) == -1 )
 	    {
 		IMAPCount->CurrentClientConnections--;
